@@ -38,7 +38,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "cat"
 
-#define AUTHORS N_ ("Torbjorn Granlund and Richard M. Stallman")
+#define AUTHORS "Torbjorn Granlund", "Richard M. Stallman"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef max
