@@ -39,7 +39,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "dd"
 
-#define AUTHORS N_ ("Paul Rubin, David MacKenzie, and Stuart Kemp")
+#define AUTHORS "Paul Rubin", "David MacKenzie", "Stuart Kemp"
 
 #ifndef SIGINFO
 # define SIGINFO SIGUSR1
@@ -650,11 +650,10 @@ apply_translations (void)
 #define MX(a) (bit_count (conversions_mask & (a)))
   if ((MX (C_ASCII | C_EBCDIC | C_IBM) > 1)
       || (MX (C_BLOCK | C_UNBLOCK) > 1)
-      || (MX (C_LCASE | C_UCASE) > 1)
-      || (MX (C_UNBLOCK | C_SYNC) > 1))
+      || (MX (C_LCASE | C_UCASE) > 1))
     {
       error (EXIT_FAILURE, 0, _("\
-only one conv in {ascii,ebcdic,ibm}, {lcase,ucase}, {block,unblock}, {unblock,sync}"));
+	only one conv in {ascii,ebcdic,ibm}, {lcase,ucase}, {block,unblock}"));
     }
 #undef MX
 
@@ -1161,7 +1160,7 @@ main (int argc, char **argv)
   atexit (close_stdout_wrapper);
 
   parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE, VERSION,
-		      AUTHORS, usage);
+		      usage, AUTHORS, (char const *) NULL);
 
   /* Don't close stdout on exit from here on.  */
   closeout_func = NULL;
