@@ -25,12 +25,11 @@
 #include "long-options.h"
 #include "error.h"
 #include "dirname.h"
-#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "dirname"
 
-#define AUTHORS N_ ("David MacKenzie and Jim Meyering")
+#define AUTHORS "David MacKenzie", "Jim Meyering"
 
 /* The name this program was run with. */
 char *program_name;
@@ -76,7 +75,7 @@ main (int argc, char **argv)
   atexit (close_stdout);
 
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
-		      AUTHORS, usage);
+		      usage, AUTHORS, (char const *) NULL);
   /* The above handles --help and --version.
      Since there is no other invocation of getopt, handle `--' here.  */
   if (argc > 1 && STREQ (argv[1], "--"))
