@@ -85,12 +85,11 @@
 
 #include "system.h"
 #include "error.h"
-#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "env"
 
-#define AUTHORS N_ ("Richard Mlynarik and David MacKenzie")
+#define AUTHORS "Richard Mlynarik", "David MacKenzie"
 
 int putenv ();
 
