@@ -27,7 +27,6 @@
 #define word unused_word_type
 
 #include "system.h"
-#include "closeout.h"
 #include "error.h"
 #include "quote.h"
 #include "xstrtol.h"
@@ -781,7 +780,7 @@ flush_paragraph (void)
 
   if (word_limit == word)
     {
-      printf ("%*s", wptr - parabuf, parabuf);
+      printf ("%*s", (int) (wptr - parabuf), parabuf);
       wptr = parabuf;
       return;
     }
