@@ -32,7 +32,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "ln"
 
-#define AUTHORS N_ ("Mike Parker and David MacKenzie")
+#define AUTHORS "Mike Parker", "David MacKenzie"
 
 #ifndef ENABLE_HARD_LINK_TO_SYMLINK_WARNING
 # define ENABLE_HARD_LINK_TO_SYMLINK_WARNING 0
@@ -73,13 +73,14 @@ int symlink ();
       {									\
 	const char *source_base;					\
 	char *tmp_source;						\
+	size_t buf_len = strlen (source) + 1;				\
 									\
-	tmp_source = (char *) alloca (strlen ((source)) + 1);		\
-	strcpy (tmp_source, (source));					\
+	tmp_source = alloca (buf_len);					\
+	memcpy (tmp_source, (source), buf_len);				\
 	strip_trailing_slashes (tmp_source);				\
 	source_base = base_name (tmp_source);				\
 									\
-	(new_dest) = (char *) alloca (strlen ((dest)) + 1		\
+	(new_dest) = alloca (strlen ((dest)) + 1			\
 				      + strlen (source_base) + 1);	\
 	stpcpy (stpcpy (stpcpy ((new_dest), (dest)), "/"), source_base);\
       }									\
@@ -110,7 +111,10 @@ static int remove_existing_files;
 /* If nonzero, list each file as it is moved. */
 static int verbose;
 
-/* If nonzero, allow the superuser to make hard links to directories. */
+/* If nonzero, allow the superuser to *attempt* to make hard links
+   to directories.  However, it appears that this option is not useful
+   in practice, since even the superuser is prohibited from hard-linking
+   directories on most (all?) existing systems.  */
 static int hard_dir_link;
 
 /* If nonzero, and the specified destination is a symbolic link to a
@@ -260,11 +264,13 @@ do_link (const char *source, const char *dest)
 
       if (backup_type != none)
 	{
+	  size_t buf_len;
 	  char *tmp_backup = find_backup_file_name (dest, backup_type);
 	  if (tmp_backup == NULL)
 	    xalloc_die ();
-	  dest_backup = (char *) alloca (strlen (tmp_backup) + 1);
-	  strcpy (dest_backup, tmp_backup);
+	  buf_len = strlen (tmp_backup) + 1;
+	  dest_backup = alloca (buf_len);
+	  memcpy (dest_backup, tmp_backup, buf_len);
 	  free (tmp_backup);
 	  if (rename (dest, dest_backup))
 	    {
@@ -357,7 +363,9 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       fputs (_("\
       --backup[=CONTROL]      make a backup of each existing destination file\n\
   -b                          like --backup but does not accept an argument\n\
-  -d, -F, --directory         hard link directories (super-user only)\n\
+  -d, -F, --directory         allow the superuser to attempt to hard link\n\
+                                directories (note: will probably fail due to\n\
+                                system restrictions, even for the superuser)\n\
   -f, --force                 remove existing destination files\n\
 "), stdout);
       fputs (_("\
