@@ -26,7 +26,7 @@
 #include "system.h"
 
 #include "md5.h"
-#include "sha.h"
+#include "sha1.h"
 #include "checksum.h"
 #include "getline.h"
 #include "error.h"
@@ -34,7 +34,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME (algorithm == ALG_MD5 ? "md5sum" : "shasum")
 
-#define AUTHORS N_ ("Ulrich Drepper and Scott Miller")
+#define AUTHORS "Ulrich Drepper", "Scott Miller"
 
 /* Most systems do not distinguish between external and internal
    text representations.  */
