@@ -314,9 +314,7 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
-#include <time.h>
 #include "system.h"
-#include "closeout.h"
 #include "error.h"
 #include "mbswidth.h"
 #include "posixver.h"
@@ -325,7 +323,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "pr"
 
-#define AUTHORS N_ ("Pete TerMaat and Roland Huebner")
+#define AUTHORS "Pete TerMaat", "Roland Huebner"
 
 #ifndef TRUE
 # define TRUE	1
@@ -462,7 +460,7 @@ static int buff_current;
 
 /* The number of characters in buff.
    Used for allocation of buff and to detect overflow of buff. */
-static int buff_allocated;
+static size_t buff_allocated;
 
 /* Array of indices into buff.
    Each entry is an index of the first character of a line.
@@ -2015,8 +2013,7 @@ store_char (int c)
   if (buff_current >= buff_allocated)
     {
       /* May be too generous. */
-      buff_allocated = 2 * buff_allocated;
-      buff = xrealloc (buff, buff_allocated * sizeof (char));
+      buff = x2nrealloc (buff, &buff_allocated, sizeof *buff);
     }
   buff[buff_current++] = (char) c;
 }
