@@ -52,9 +52,9 @@
 #include <getopt.h>
 
 #include "system.h"
+#include "c-strtod.h"
 #include "long-options.h"
 #include "error.h"
-#include "closeout.h"
 #include "unicodeio.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
@@ -62,12 +62,6 @@
 
 #define AUTHORS "David MacKenzie"
 
-#ifndef STDC_HEADERS
-double strtod ();
-long int strtol ();
-unsigned long int strtoul ();
-#endif
-
 #define isodigit(c) ((c) >= '0' && (c) <= '7')
 #define hextobin(c) ((c) >= 'a' && (c) <= 'f' ? (c) - 'a' + 10 : \
 		     (c) >= 'A' && (c) <= 'F' ? (c) - 'A' + 10 : (c) - '0')
@@ -195,7 +189,7 @@ FUNC_NAME (s)								 \
 
 STRTOX (unsigned long int, xstrtoul, (strtoul (s, &end, 0)))
 STRTOX (long int,          xstrtol,  (strtol  (s, &end, 0)))
-STRTOX (double,            xstrtod,  (strtod  (s, &end)))
+STRTOX (double,            xstrtod,  (c_strtod (s, &end)))
 
 /* Output a single-character \ escape.  */
 
@@ -567,7 +561,7 @@ main (int argc, char **argv)
   posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
   if (!posixly_correct)
     parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
-			AUTHORS, usage);
+			usage, AUTHORS, (char const *) NULL);
 
   /* The above handles --help and --version.
      Since there is no other invocation of getopt, handle `--' here.  */
