@@ -1,5 +1,5 @@
 /* tail -- output the last part of file(s)
-   Copyright (C) 1989, 90, 91, 1995-2003 Free Software Foundation, Inc.
+   Copyright (C) 1989, 90, 91, 1995-2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 
 #include "system.h"
 #include "argmatch.h"
+#include "c-strtod.h"
 #include "error.h"
 #include "inttostr.h"
 #include "posixver.h"
@@ -46,7 +47,7 @@
 #define PROGRAM_NAME "tail"
 
 #define AUTHORS \
-  N_ ("Paul Rubin, David MacKenzie, Ian Lance Taylor, and Jim Meyering")
+  "Paul Rubin", "David MacKenzie, Ian Lance Taylor", "Jim Meyering"
 
 #ifndef ENOSYS
   /* Some systems don't have ENOSYS -- this should be a big enough
@@ -768,9 +769,9 @@ start_bytes (const char *pretty_filename, int fd, uintmax_t n_bytes,
 	n_bytes -= bytes_read;
       else
 	{
-	  size_t remainder = bytes_read - n_bytes;
-	  if (remainder)
-	    xwrite (STDOUT_FILENO, &buffer[n_bytes], remainder);
+	  size_t n_remaining = bytes_read - n_bytes;
+	  if (n_remaining)
+	    xwrite (STDOUT_FILENO, &buffer[n_bytes], n_remaining);
 	  break;
 	}
     }
@@ -1610,7 +1611,7 @@ parse_options (int argc, char **argv,
 	case 's':
 	  {
 	    double s;
-	    if (xstrtod (optarg, NULL, &s) || ! (0 <= s))
+	    if (xstrtod (optarg, NULL, &s, c_strtod) || ! (0 <= s))
 	      error (EXIT_FAILURE, 0,
 		     _("%s: invalid number of seconds"), optarg);
 	    *sleep_interval = s;
@@ -1698,7 +1699,6 @@ main (int argc, char **argv)
 	--n_units;
     }
 
-
   if (optind < argc)
     {
       n_files = argc - optind;
@@ -1711,6 +1711,25 @@ main (int argc, char **argv)
       file = &dummy_stdin;
     }
 
+  {
+    bool found_hyphen = false;
+
+    for (i = 0; i < n_files; i++)
+      if (STREQ (file[i], "-"))
+	found_hyphen = true;
+
+    /* When following by name, there must be a name.  */
+    if (found_hyphen && follow_mode == Follow_name)
+      error (EXIT_FAILURE, 0, _("cannot follow %s by name"), quote ("-"));
+
+    /* When following forever, warn if any file is `-'.
+       This is only a warning, since tail's output (before a failing seek,
+       and that from any non-stdin files) might still be useful.  */
+    if (forever && found_hyphen && isatty (STDIN_FILENO))
+      error (0, 0, _("warning: following standard input"
+		     " indefinitely is ineffective"));
+  }
+
   F = xmalloc (n_files * sizeof (F[0]));
   for (i = 0; i < n_files; i++)
     F[i].name = file[i];
