@@ -29,7 +29,6 @@
 #include <sys/types.h>
 
 #include "system.h"
-#include "closeout.h"
 #include "long-options.h"
 #include "error.h"
 #include "readtokens.h"
@@ -112,7 +111,7 @@ With no FILE, or when FILE is -, read standard input.\n\
 static struct item *
 new_item (const char *str)
 {
-  struct item *k = xmalloc (sizeof (struct item));
+  struct item *k = xmalloc (sizeof *k);
 
   k->str = (str ? xstrdup (str): NULL);
   k->left = k->right = NULL;
@@ -287,7 +286,7 @@ record_relation (struct item *j, struct item *k)
   if (!STREQ (j->str, k->str))
     {
       k->count++;
-      p = xmalloc (sizeof (struct successor));
+      p = xmalloc (sizeof *p);
       p->suc = k;
       p->next = j->top;
       j->top = p;
@@ -561,7 +560,7 @@ main (int argc, char **argv)
   exit_status = 0;
 
   parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE, VERSION,
-		      AUTHORS, usage);
+		      usage, AUTHORS, (char const *) NULL);
 
   while ((opt = getopt_long (argc, argv, "", long_options, NULL)) != -1)
     switch (opt)
