@@ -41,7 +41,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "closeout.h"
 #include "error.h"
 #include "posixver.h"
 
@@ -77,6 +76,7 @@ static int tab_size;
    after `tab_list' is exhausted, the rest of the line is printed
    unchanged.  The first column is column 0. */
 static int *tab_list;
+static size_t n_tabs_allocated;
 
 /* The index of the first invalid element of `tab_list',
    where the next element can be added. */
@@ -122,9 +122,8 @@ add_tabstop (int tabval)
 {
   if (tabval == -1)
     return;
-  if (first_free_tab % TABLIST_BLOCK == 0)
-    tab_list = (int *) xrealloc ((char *) tab_list,
-				 first_free_tab + TABLIST_BLOCK);
+  if (first_free_tab == n_tabs_allocated)
+    tab_list = x2nrealloc (tab_list, &n_tabs_allocated, sizeof *tab_list);
   tab_list[first_free_tab++] = tabval;
 }
 
@@ -234,9 +233,10 @@ unexpand (void)
   int tab_index = 0;		/* For calculating width of pending tabs. */
   int print_tab_index = 0;	/* For printing as many tabs as possible. */
   unsigned int column = 0;	/* Column on screen of next char. */
-  int next_tab_column; 		/* Column the next tab stop is on. */
+  int next_tab_column;		/* Column the next tab stop is on. */
   int convert = 1;		/* If nonzero, perform translations. */
   unsigned int pending = 0;	/* Pending columns of blanks. */
+  int saved_errno;
 
   fp = next_file ((FILE *) NULL);
   if (fp == NULL)
@@ -248,6 +248,7 @@ unexpand (void)
   for (;;)
     {
       c = getc (fp);
+      saved_errno = errno;
 
       if (c == ' ' && convert && column < TAB_STOP_SENTINEL)
 	{
@@ -327,6 +328,7 @@ unexpand (void)
 
 	  if (c == EOF)
 	    {
+	      errno = saved_errno;
 	      fp = next_file (fp);
 	      if (fp == NULL)
 		break;		/* No more files. */
@@ -409,6 +411,7 @@ main (int argc, char **argv)
 
   bool obsolete_tablist = false;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -483,10 +486,7 @@ main (int argc, char **argv)
       tab_size = 0;
     }
 
-  if (optind == argc)
-    file_list = stdin_argv;
-  else
-    file_list = &argv[optind];
+  file_list = (optind < argc ? &argv[optind] : stdin_argv);
 
   unexpand ();
 
