@@ -31,12 +31,11 @@
 #include "error.h"
 #include "long-options.h"
 #include "readutmp.h"
-#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "uptime"
 
-#define AUTHORS N_ ("Joseph Arceneaux, David MacKenzie, and Kaveh Ghazi")
+#define AUTHORS "Joseph Arceneaux", "David MacKenzie", "Kaveh Ghazi"
 
 int getloadavg ();
 
@@ -217,7 +216,7 @@ main (int argc, char **argv)
   atexit (close_stdout);
 
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
-		      AUTHORS, usage);
+		      usage, AUTHORS, (char const *) NULL);
 
   while ((optc = getopt_long (argc, argv, "", longopts, &longind)) != -1)
     {
