@@ -71,7 +71,7 @@ extern int wcwidth ();
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "wc"
 
-#define AUTHORS N_ ("Paul Rubin and David MacKenzie")
+#define AUTHORS "Paul Rubin", "David MacKenzie"
 
 /* Size of atomic reads. */
 #define BUFFER_SIZE (16 * 1024)
@@ -674,6 +674,8 @@ main (int argc, char **argv)
 		      max_line_length, _("total"));
     }
 
+  free (fstatus);
+
   if (have_read_stdin && close (STDIN_FILENO) != 0)
     error (EXIT_FAILURE, errno, "-");
 
